# coding: utf-8

"""
    FlexPrice API

    FlexPrice API Service

    The version of the OpenAPI document: 1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class TypesSubscriptionStatus(str, Enum):
    """
    TypesSubscriptionStatus
    """

    """
    allowed enum values
    """
    ACTIVE = 'active'
    PAUSED = 'paused'
    CANCELLED = 'cancelled'
    INCOMPLETE = 'incomplete'
    INCOMPLETE_EXPIRED = 'incomplete_expired'
    PAST_DUE = 'past_due'
    TRIALING = 'trialing'
    UNPAID = 'unpaid'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of TypesSubscriptionStatus from a JSON string"""
        return cls(json.loads(json_str))


